#!/usr/local/bin/perl
##################################################################
# @(#) $Id: mrtg-ping-probe,v 2.3 2001/01/21 20:01:16 pwo Exp $
# @(#) mrtg-ping-probe release $Name: Release_2_0_2 $
#
# Copyright (c) 1997-2001 Peter W. Osel <pwo@guug.de>.
# All Rights Reserved.
#
# See the file COPYRIGHT in the distribution for the exact terms.
#
##################################################################
#
# "I saw -- from the cathedral -- you were watching me"
#
#	-- Tanita Tikaram, `Cathedral Song'
#	on: Tanita Tikaram, `Ancient Heart', 1988, WEA Records
#
##################################################################
#
# Modified from the original for use by cacti
# Any questions or comments regarding this code should be directed to:              
#  - sidewinder at shitworks.com
#
##################################################################

use Config;
use XML::Simple;
use Mysql;
use Net::Syslog;


  $aloe_config_file = 'C:\\inetpub\\wwwroot\\cacti\\aloe\\config\\aloe-config.xml';

  #
  # Load the configuration settings from aloe.xml.

  if (open (my $afh, "<", $aloe_config_file)) {

  	close($afh);
	$aloe  = XMLin($aloe_config_file);

  } else { die "Could not find $aloe_config_file, script can not continue"; }


  my $debug		= $aloe->{settings}->{debug}->{value};
  my $syslogHost	= $aloe->{settings}->{aloeLogHost}->{value};
  my $syslog_facility   = $aloe->{settings}->{aloeLogFacility}->{value};

  my $aloedbhost	= $aloe->{settings}->{aloeDBHost}->{value};
  my $aloedb		= $aloe->{settings}->{aloeDB}->{value};
  my $aloedbuser	= $aloe->{settings}->{aloeDBUser}->{value};
  my $aloedbpswd	= $aloe->{settings}->{aloeDBPswd}->{value};

  my $cactidbhost	= $aloe->{settings}->{cactiDBHost}->{value};
  my $cactidb		= $aloe->{settings}->{cactiDB}->{value};
  my $cactidbuser	= $aloe->{settings}->{cactiDBUser}->{value};
  my $cactidbpswd	= $aloe->{settings}->{cactiDBPswd}->{value};


  our $config_path      = $aloe->{settings}->{configPath}->{value};

  my @script_name       = split m!\\!, $0;
  my $event_source      = "cacti -> $script_name[((scalar(@script_name)) - 1)]";

  my $cdbh = Mysql->connect(undef, $cactidb, $cactidbuser, $cactidbpswd);
  my $cdbh = Mysql->connect($cactidbhost, $cactidb, $cactidbuser, $cactidbpswd);

  #
  # find the rrd file associated with this script instance

  my $dbquery = join('',"select rrd_path from data_input_data_cache where command like '%$script_name[((scalar(@script_name)) - 1)]", " ", @ARGV[0], "';");

  my $csth = $cdbh->query($dbquery);
  my $cdbr = $csth->fetchrow();

  if ($debug) {  print "query  -> $dbquery \n"; print "result -> $cdbr \n"; }

  our $config_template          = join('', $config_path, "ping-probe-config-template.xml");
  our $config_specific_template = join('', $cdbr, ".xml" );

  if ($debug) {  print "ping-probe output file -> $config_specific_template \n"; }


  #
  # Try to load configuration parameters from the xml file specific for this script instance
  # If that doesn't work, assume it is the first time the script has run for this script instance and generate one.


  if (!loadConfigSpecificTemplate()) {

	if ($debug) { print "loading default values\n"; }

	loadConfigTemplate();
	outputToXMLFile();
	loadConfigSpecificTemplate();
  }


  # Check Sanity of Arguments
  $err += check_numeric("-d", $DeadTime);
  $err += check_numeric("-k", $PacketCount);
  $err += check_numeric("-l", $PacketLength);
  $err += check_numeric("-t", $TimeOut);
  $err += check_picklst("-p", $PickList);


  ($HostToPing) = @ARGV[0];


  ($pt{min}, $pt{avg}, $pt{max}, $pt{loss}) =
  	ping($HostToPing, $PacketLength, $PacketCount, $TimeOut);

  my  $lastUpdate = time();
  my  $cooked_lastUpdate = localtime($lastUpdate);

  print "$Prog_name: DBG: main(): ping() ret: $pt{min}, $pt{avg}, $pt{max}, $pt{loss}\n" if $debug;

  $PickList =~ /^(?:(\d+)\*)?(\w+)\/(?:(\d+)\*)?(\w+)$/;
  ($f1, $p1, $f2, $p2) = ($1, $2, $3, $4);
  $f1 = $f1 ? $f1 : 1;
  $f2 = $f2 ? $f2 : 1;

  print "$Prog_name: DBG: main(): before psub: f = ($f1 $f2); p = ($p1 $p2)\n" if $debug;

  $p1 = $p1 =~ /^\d+$/ ? $p1 : $pt{$p1};
  $p2 = $p2 =~ /^\d+$/ ? $p2 : $pt{$p2};

  print "$Prog_name: DBG: main(): after  psub: f = ($f1 $f2); p = ($p1 $p2)\n" if $debug;


  $pAvg  = $f1 * $p1;
  $pLoss = $f2 * $p2;


  $cooked_avg_bg  = "Good";
  $cooked_loss_bg = "Good";

  if (($pAvg > $rtt_warn_threshold) && ($pAvg <= $rtt_error_threshold)) { logEvent("Average RTT now at $pAvg ms", "warning", $HostToPing); $cooked_avg_bg = "Warning"; }
  	elsif ($pAvg > $rtt_error_threshold) { logEvent("Average RTT now at $pAvg ms", "err", $HostToPing); $cooked_avg_bg = "Error";}

  if (($pLoss > 0) && ($pLoss <= $pktLoss_warn_threshold)) { logEvent("Packet Loss now at $pLoss%", "warning", $HostToPing); $cooked_loss_bg = "Warning";}
	elsif ($pLoss > $pktLoss_error_threshold) { logEvent("Packet Loss now at $pLoss%", "err", $HostToPing); $cooked_loss_bg = "Error"; }

  #
  # Output values to cacti

  printf "avg:%d loss:%d", $f1 * $p1, $f2 * $p2;

  #
  # Output values to xml file

    outputToXMLFile();

exit(0);


##################################################################
sub check_numeric {
	my($opt, $val) = @_;
	my($err) = 0;

	unless ($val =~ /^\d+$/) {
		print STDERR "$Prog_name: FATAL: option $opt requires numeric argument.\n";
		++$err;
		}
	return($err);
	}

##################################################################
sub check_picklst {
	my($opt, $val) = @_;
	my($err, $i) = (0, 0);
	my(@v, @n);

	unless ($val =~ /^(?:(\d+)\*)?(\w+)\/(?:(\d+)\*)?(\w+)$/) {
		print STDERR "$Prog_name: FATAL: option $opt requires [factor*]word/[factor*]word argument, I found \"$val\"\n";
		++$err;
		}
	@n = ($1, $3);
	@v = ($2, $4);

	print "$Prog_name: DBG: check_picklst(): n = (@n); v = (@v)\n" if $debug;

	foreach $i (0..1) {
		unless ($v[$i] =~ /^(min|max|avg|loss|\d+)$/) {
			print STDERR "$Prog_name: FATAL: option $opt uses unknown item $v[$i]\n";
			print STDERR "$Prog_name: FATAL: option $opt may choose from min|max|avg|loss or number\n";
			++$err;
			}
		}

	return($err);
	}

##################################################################
# ping selects the external or internal, builtin ping method
# and starts the external ping program optionally with a
# timout to abort external ping programs that seem to hang
# when the target is unreachable.

sub ping {
	my($host, $length, $count, $timeout) = @_;
	my(%pt);

	($pt{min}, $pt{avg}, $pt{max}, $pt{loss}) =
		ext_ping($host, $length, $count, $timeout);

	return($pt{min}, $pt{avg}, $pt{max}, $pt{loss});
	}

##################################################################
# ping host retrieve and return min, avg, max round trip time
# relying on finding a standard ping in PATH.
# Try to not be platform specific if at all possible.
#
sub ext_ping {
	my($host, $length, $count, $timeout) = @_;
	my(%ping, $ping_output, $redirect_stderr, $pid, %pt);
	my($alarm_exists);


	# List of known ping programs
	%ping = (
		'MSWin32' =>	"ping -l $length -n $count $host",
		'aix'	=>	"/etc/ping $host $length $count",
		'bsdos'	=>	"/bin/ping -s $length -c $count $host",
		'freebsd' =>	"/sbin/ping -s $length -c $count $host",
		'hpux'	=>	"/etc/ping $host $length $count",
		'irix'	=>	"/usr/etc/ping -c $count -s $length $host",
		'linux'	=>	"/bin/ping -c $count -s $length $host",
		'netbsd' =>	"/sbin/ping -s $length -c $count $host",
		'openbsd' =>	"/sbin/ping -s $length -c $count $host",
		'os2' =>	"ping $host $length $count",
		'OS/2' =>	"ping $host $length $count",
		'dec_osf'=>	"/sbin/ping -s $length -c $count $host",
		'solaris' =>	"/usr/sbin/ping -s $host $length $count",
		'sunos'	=>	"/usr/etc/ping -s $host $length $count",
		);

	unless (defined($ping{$Config{'osname'}})) {
		print STDERR "${Prog_name}: FATAL: Not yet configured for $Config{'osname'}\n";
		exit(1);
		}

	# add ping options, if any
	$ping{$Config{'osname'}} =~ s/ / $PingOptions / if $PingOptions;

	# windows 95/98 does not support stderr redirection...
	# also OS/2 users reported problems with stderr redirection...
	$redirect_stderr = $Config{'osname'} =~ /^(MSWin32|os2|OS\/2)$/i ? "" : "2>&1";

	# initialize return values
	$pt{loss} = 100;
	$pt{min} = $pt{avg} = $pt{max} = $DeadTime;

	unless ($pid = open(PING, "$ping{$Config{'osname'}} $redirect_stderr |")) {
		print STDERR "${Prog_name}: FATAL: Can't open $ping{$Config{'osname'}}: $!";
		exit(1);
		}

	$alarm_exists =  eval { alarm(0); 1 };
	print STDERR "${Prog_name}: WARN: builtin alarm() does not exist, can't timout ping command\n"
		if $Verbose && $timeout && !$alarm_exists;

	if ($alarm_exists) {
		# read and timeout ping() if it takes too long...
		eval {
			local $SIG{ALRM} = sub { die "alarm\n" };	# \n required!
			alarm $timeout;
			while (<PING>) {
				$ping_output .= $_;
				}
			alarm 0;
			};

		if ($@) {
			die unless $@ eq "alarm\n";	# propagate unexpected errors
			# timed out, kill child, get remaining output, ...
			kill $pid;
			while (<PING>) {
				$ping_output .= $_;
				}
			close(PING);

			# ... and set return values to dead values
			unless ($Silent) {
				print STDERR "${Prog_name}: ERROR: external ping hit timeout $timeout, assuming target $host is unreachable\n";
				print STDERR "${Prog_name}: INFO: The output of the ping command $ping{$Config{'osname'}} was:\n";
				print STDERR "$ping_output\n";
				}
			return($pt{min}, $pt{avg}, $pt{max}, $pt{loss});
			}
		}
	else {
		# read and hope that ping() will return in time...
		while (<PING>) {
			$ping_output .= $_;
			}

		}

	# didn't time out, analyse ping output.
	close(PING);

	# try to find round trip times
	if ($ping_output =~ m@round-trip(?:\s+\(ms\))?\s+min/avg/max(?:/(?:m|std)-?dev)?\s+=\s+(\d+(?:\.\d+)?)/(\d+(?:\.\d+)?)/(\d+(?:\.\d+)?)@m) {
		$pt{min} = $1; $pt{avg} = $2; $pt{max} = $3;
		}
	elsif ($ping_output =~ m@^\s+\w+\s+=\s+(\d+(?:\.\d+)?)ms,\s+\w+\s+=\s+(\d+(?:\.\d+)?)ms,\s+\w+\s+=\s+(\d+(?:\.\d+)?)ms\s+$@m) {
		# this should catch most windows locales
		$pt{min} = $1; $pt{avg} = $3; $pt{max} = $2;
		}
	else {
		unless ($Silent) {
			print STDERR "${Prog_name}: ERROR: Could not find ping summary for $host\n";
			print STDERR "${Prog_name}: INFO: The output of the ping command $ping{$Config{'osname'}} was:\n";
			print STDERR "$ping_output\n";
			}
		}

	# try to find packet loss
	# ToDo: only if requested?)
	if ($ping_output =~ m@(\d+)% packet loss$@m) {
		# Unix
		$pt{loss} = $1;
		}
	elsif ($ping_output =~ m@\(perte\s+(\d+)%\),\s+$@m) {
		# Windows french locale
		$pt{loss} = $1;
		}
	elsif ($ping_output =~ m@\((\d+)%\s+(?:loss|perdidos)\),\s+$@m) {
		# Windows portugesee, spanish locale
		$pt{loss} = $1;
		}
	else {
		unless ($Silent) {
			print STDERR "${Prog_name}: ERROR: Could not find packet loss summary for $host\n";
			print STDERR "${Prog_name}: INFO: The output of the ping command $ping{$Config{'osname'}} was:\n";
			print STDERR "$ping_output\n";
			}
		}

	return($pt{min}, $pt{avg}, $pt{max}, $pt{loss});
}


sub loadConfigSpecificTemplate($$) {

  if (open (my $cfh, "<", $config_specific_template)) {

  	close($cfh);
	$cfgParams  = XMLin($config_specific_template);

	if (defined($cfgParams->{config}->{cDebug}->{value}))		{ $debug	= "$cfgParams->{config}->{cDebug}->{value}";		}
	if (defined($cfgParams->{config}->{cSilent}->{value}))		{ $Silent	= "$cfgParams->{config}->{cSilent}->{value}";		}
	if (defined($cfgParams->{config}->{cDeadTime}->{value}))	{ $DeadTime	= "$cfgParams->{config}->{cDeadTime}->{value}";		}
	if (defined($cfgParams->{config}->{cPacketCount}->{value}))	{ $PacketCount	= "$cfgParams->{config}->{cPacketCount}->{value}";	}
	if (defined($cfgParams->{config}->{cPacketLength}->{value}))	{ $PacketLength	= "$cfgParams->{config}->{cPacketLength}->{value}";	}
	if (defined($cfgParams->{config}->{cPickList}->{value}))	{ $PickList	= "$cfgParams->{config}->{cPickList}->{value}";		}
	if (defined($cfgParams->{config}->{cPingOptions}->{value}))	{ $PingOptions	= "$cfgParams->{config}->{cPingOptions}->{value}";	}
	if (defined($cfgParams->{config}->{cTimeOut}->{value}))		{ $TimeOut	= "$cfgParams->{config}->{cTimeOut}->{value}";		}

	if (defined($cfgParams->{threshold}->{trtt}->{warn}))		{ $rtt_warn_threshold       = "$cfgParams->{threshold}->{trtt}->{warn}";		}
	if (defined($cfgParams->{threshold}->{trtt}->{error}))		{ $rtt_error_threshold      = "$cfgParams->{threshold}->{trtt}->{error}";		}
	if (defined($cfgParams->{threshold}->{tpktLoss}->{warn}))	{ $pktLoss_warn_threshold   = "$cfgParams->{threshold}->{tpktLoss}->{warn}";		}
	if (defined($cfgParams->{threshold}->{tpktLoss}->{error}))	{ $pktLoss_error_threshold  = "$cfgParams->{threshold}->{tpktLoss}->{error}";		}

  }  else { return 0; }

  return 1;


} # loadConfigSpecificTemplate

sub loadConfigTemplate($$) {

  if (open (my $cfh, "<", $config_template)) {

  	close($cfh);
	$cfgParams  = XMLin($config_template);

	if (defined($cfgParams->{config}->{cDebug}->{value}))		{ $debug	= "$cfgParams->{config}->{cDebug}->{value}";		}
	if (defined($cfgParams->{config}->{cSilent}->{value}))		{ $Silent	= "$cfgParams->{config}->{cSilent}->{value}";		}
	if (defined($cfgParams->{config}->{cDeadTime}->{value}))	{ $DeadTime	= "$cfgParams->{config}->{cDeadTime}->{value}";		}
	if (defined($cfgParams->{config}->{cPacketCount}->{value}))	{ $PacketCount	= "$cfgParams->{config}->{cPacketCount}->{value}";	}
	if (defined($cfgParams->{config}->{cPacketLength}->{value}))	{ $PacketLength	= "$cfgParams->{config}->{cPacketLength}->{value}";	}
	if (defined($cfgParams->{config}->{cPickList}->{value}))	{ $PickList	= "$cfgParams->{config}->{cPickList}->{value}";		}
	if (defined($cfgParams->{config}->{cPingOptions}->{value}))	{ $PingOptions	= "$cfgParams->{config}->{cPingOptions}->{value}";	}
	if (defined($cfgParams->{config}->{cTimeOut}->{value}))		{ $TimeOut	= "$cfgParams->{config}->{cTimeOut}->{value}";		}

	if (defined($cfgParams->{threshold}->{trtt}->{warn}))		{ $rtt_warn_threshold       = "$cfgParams->{threshold}->{trtt}->{warn}";		}
	if (defined($cfgParams->{threshold}->{trtt}->{error}))		{ $rtt_error_threshold      = "$cfgParams->{threshold}->{trtt}->{error}";		}
	if (defined($cfgParams->{threshold}->{tpktLoss}->{warn}))	{ $pktLoss_warn_threshold   = "$cfgParams->{threshold}->{tpktLoss}->{warn}";		}
	if (defined($cfgParams->{threshold}->{tpktLoss}->{error}))	{ $pktLoss_error_threshold  = "$cfgParams->{threshold}->{tpktLoss}->{error}";		}

  }   else { die "Could not find $config_template, script can not continue"; }


} # loadConfigTemplate


sub logEvent($$) {

  # Valid Facilities are:  
  #  kern, user, mail, daemon, auth, syslog, lpr, news, uucp, cron,  
  #  authpriv, ftp, local0, local1, local2, local3, local4, local5, local6 

  # Valid Priorities are:  
  #  emerg, alert, crit, err, warning, notice, info, debug 

  my $event_message  = $_[0];
  my $event_priority = $_[1];
  my $management_ip  = $_[2];

  my $log_message =  "-> $management_ip $event_message";


  if ($aloe->{settings}->{aloeSysLogMethod}->{value}) {

	  my $log_message =  "-> $management_ip $snmpResult->{$ifDescr} $event_message";

	  my $s=new Net::Syslog(Facility=>$syslog_facility, Priority=>'debug', SyslogHost => $syslogHost, Name=> $event_source);
	  $s->send($log_message,Priority=>$event_priority);
  }


  if ($aloe->{settings}->{aloeDBLogMethod}->{value}) {

	  my $log_message =  "$event_source -> $management_ip $snmpResult->{$ifDescr} $event_message";	

  	  my $dbquery = "insert into logs (date, time, priority, facility, host, msg, program) values (now(), now(), '$event_priority', '$syslog_facility', 'localhost', '$log_message', '$event_source')"; 

	  my $adbh = Mysql->connect(undef, $aloedb, $aloedbuser, $aloedbpswd);
	  my $adbh = Mysql->connect($aloedbhost, $aloedb, $aloedbuser, $aloedbpswd);

	  my $asth = $adbh->query($dbquery);

  }


  if ($aloe->{settings}->{aloeSMTPLogMethod}->{value}) {

	  my $log_message =  "$event_source -> $management_ip $snmpResult->{$ifDescr} $event_message";

	    use Net::SMTP;
	    $smtp = Net::SMTP->new($aloe->{settings}->{aloeSMTPRelay}->{value});
	    $smtp->mail($aloe->{settings}->{aloeSMTPFromAddress}->{value});
	    $smtp->to($aloe->{settings}->{aloeSMTPRecipient}->{value});
	    $smtp->data();
	    $smtp->datasend("To: $aloe->{settings}->{aloeSMTPRecipient}->{value} \n");
	    $smtp->datasend("Subject: $event_priority, $log_message\n");
	    $smtp->datasend("\n");
	    $smtp->datasend($event_priority);
	    $smtp->datasend("\n");
	    $smtp->datasend($log_message);
	    $smtp->dataend();
	    $smtp->quit;

  } 


} # logEvent


sub outputToXMLFile($$) {

  #
  # update xml file with delta and display values.


  open ($dfh, ">", $config_specific_template);

print {$dfh} <<"END";
<aloexml>
	<name>Ping Probe</name>
	<description>$script_name[((scalar(@script_name)) - 1)] Measures the RTT to a host using ping</description>
	<refreshvalue>$0 @ARGV[0]</refreshvalue>

	<config>
		<cDebug>
			<desc>Turn debug on or off (0=Off, 1=On)</desc>
			<value>$debug</value>
		</cDebug>
		<cSilent>
			<desc>Do not generate error messages if there is no response from the ping program or if it ran into the timeout. (0=Off, 1=On)</desc>
			<value>$Silent</value>
		</cSilent>
		<cDeadTime>
			<desc>Specifies the value returned for round trip times in case we assume that the target is down.</desc>
			<value>$DeadTime</value>
		</cDeadTime>
		<cPacketCount>
			<desc>Specifies the number of ping packets to be sent.</desc>
			<value>$PacketCount</value>
		</cPacketCount>
		<cPacketLength>
			<desc>Specifies the size of the ping packets that are sent.</desc>
			<value>$PacketLength</value>
		</cPacketLength>
		<cPingOptions>
			<desc>Specifies any additional options to be sent to the ping program. 0 to send none</desc>
			<value>$PingOptions</value>
		</cPingOptions>
		<cPickList>
			<desc>Specifies the option values that are passed back to cacti.</desc>
			<value>$PickList</value>
		</cPickList>
		<cTimeOut>
			<desc>Abort the external ping program after timeout seconds.</desc>
			<value>$TimeOut</value>
		</cTimeOut>

	</config>

	<threshold>
		<trtt>
			<desc>Specify the threshold value (in ms) for round trip time</desc>
			<warn>$rtt_warn_threshold</warn>
			<error>$rtt_error_threshold</error>
		</trtt>
		<tpktLoss>
			<desc>Specify the threshold value (in %) for packet loss</desc>
			<warn>$pktLoss_warn_threshold</warn>
			<error>$pktLoss_error_threshold</error>
		</tpktLoss>
	</threshold>

	<displayfields>
		<dtitle>
			<colHeader>$HostToPing</colHeader>
		</dtitle>
		<dMinimum>
			<heading>Minimum RTT</heading>
			<value>$pt{min}</value>
		</dMinimum>
		<dMaximum>
			<heading>Maximum RTT</heading>
			<value>$pt{max}</value>
		</dMaximum>
		<dAvgerage>
			<heading>Average RTT</heading>
			<value>
				<rttavg> $pt{avg}
				<bgcolor>$cooked_avg_bg</bgcolor>
				</rttavg>
			</value>
		</dAvgerage>
		<dpktLoss>
			<heading>% Packet Loss</heading>
			<value>
				<ploss> $pt{loss}
				<bgcolor>$cooked_loss_bg</bgcolor>
				</ploss>
			</value>
		</dpktLoss>
		<dlastUpdate>
			<heading>Refreshed</heading>
			<value>$cooked_lastUpdate</value>
		</dlastUpdate>
	</displayfields>
</aloexml>

END

close ($dfh);

} # outputToXMLFile


__END__

